/**
 * Test a high character to make sure signedness 
 * isn't messing us up.
 */

#include <stdio.h>

struct Fsm
{
	int cs;

	// Initialize the machine. Invokes any init statement blocks. Returns 0
	// if the machine begins in a non-accepting state and 1 if the machine
	// begins in an accepting state.
	int init( );

	// Execute the machine on a block of data. Returns -1 if after processing
	// the data, the machine is in the error state and can never accept, 0 if
	// the machine is in a non-accepting state and 1 if the machine is in an
	// accepting state.
	int execute( const unsigned char *data, int len );

	// Indicate that there is no more data. Returns -1 if the machine finishes
	// in the error state and does not accept, 0 if the machine finishes
	// in any other non-accepting state and 1 if the machine finishes in an
	// accepting state.
	int finish( );
};

%%{
	machine Fsm;

	alphtype unsigned char;

	# Indicate we got the high character.
	action gothigh {
		printf("yes\n");
	}

	main := 0xe8 @gothigh '\n';
}%%

%% write data;

int Fsm::init( )
{
	%% write init;
	return 0;
}

int Fsm::execute( const unsigned char *_data, int _len )
{
	const unsigned char *p = _data;
	const unsigned char *pe = _data+_len;
	%% write exec;
	if ( cs == Fsm_error )
		return -1;
	if ( cs >= Fsm_first_final )
		return 1;
	return 0;
}

int Fsm::finish()
{
	%% write eof;
	if ( cs == Fsm_error )
		return -1;
	if ( cs >= Fsm_first_final )
		return 1;
	return 0;
}

#include <stdio.h>
#define BUFSIZE 2048

Fsm fsm;
unsigned char buf[BUFSIZE];

int main()
{
	fsm.init();
	while ( 1 ) {
		int len = fread( buf, 1, BUFSIZE, stdin );
		fsm.execute( buf, len );
		if ( len != BUFSIZE )
			break;
	}
	if ( fsm.finish() > 0 )
		printf("ACCEPT\n");
	else
		printf("FAIL\n");
	return 0;
}

